unit filbind;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms,
  Dialogs;

type
  TFileBinder = class(TComponent)
  private
    { Private declarations }
    FFil:     String;
    FLen:     Longint;
    Data:     PChar;
    procedure SetFName(Value:String);
    procedure Ignore(Value:Longint);
    procedure DefineProperties(Filer:TFiler); Override;
  protected
    { Protected declarations }
  public
    { Public declarations }
    Constructor Create(AOwner:TComponent); Override;
    Destructor Destroy; Override;
    procedure SaveToFile(Filen:String);
    procedure WriteSpecData(Writer:TStream);
    procedure ReadSpecData(Reader:TStream);
  published
    { Published declarations }
    property Filename:      String read FFil write SetFName;
    property FileLength:    Longint read FLen write Ignore;
  end;

procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('System', [TFileBinder]);
end;

procedure TFileBinder.DefineProperties(Filer:TFiler); 
begin
     Inherited DefineProperties(Filer);
     Filer.DefineBinaryProperty('Data',ReadSpecData,WriteSpecData,Data <> nil);
end;

procedure TFileBinder.SetFName(Value:String);
var
F:        File;
I:        Integer;
begin
    If Value = '' then begin
        FLen := 0;
        FFil := '';
        FreeMem(Data);
        GetMem(Data,1); //use only 1 byte, effectively freeing up the memory
        Exit;
     end;
     If not (csDesigning in ComponentState) then begin
        FFil := Value;
        Exit;
     end;
     If not FileExists(Value) then begin
//        if (csDesigning in ComponentSpecData) and
//           Not (csReading in ComponentSpecData) then ShowMessage('Couldn''t load file '+Value+'!');
//        FFil := '';
        Exit;
     end;
     AssignFile(F,Value); Reset(F,1);
     I:=FileSize(F); FreeMem(Data); GetMem(Data,I);
     BlockRead(F,Data^,I);
     CloseFile(F);
     FLen := I; FFil := Value;
end;

Constructor TFileBinder.Create(AOwner:TComponent);
begin
     Inherited Create(AOwner);
     GetMem(Data,1); FLen := 0;
end;

Destructor TFileBinder.Destroy;
begin
     FreeMem(Data);
     Inherited Destroy;
end;

procedure TFileBinder.Ignore(Value:Longint);
begin end;

procedure TFileBinder.SaveToFile(Filen:String);
var
F: File;
begin
AssignFile(F,Filen); Rewrite(F,1);
BlockWrite(F,Data^,FLen);
CloseFile(F);
end;

procedure TFileBinder.WriteSpecData(Writer:TStream);
begin
Writer.Write(FLen,4);
Writer.Write(Data^,FLen);
end;

procedure TFileBinder.ReadSpecData(Reader:TStream);
begin
Reader.Read(FLen,4);
GetMem(Data,FLen);
Reader.Read(Data^,FLen);
end;

end.
